package com.android.launcher3;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.Resources;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.provider.MediaStore;
import android.util.DisplayMetrics;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.android.launcher3.accessibility.ListenerAppService;
import com.android.launcher3.manager.ColumnDef;
import com.android.launcher3.manager.LauncherManager;
import com.android.launcher3.model.ConfigManager;
import com.android.launcher3.util.Logger;
import com.android.launcher3.util.PermissionUtil;
import com.android.launcher3.util.TimeUtils;
import com.android.launcher3.views.SwipeUpFinishLayout;

import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

/**
 * 锁屏
 */
public class LockScreenActivity extends BaseActivity implements View.OnClickListener {

    public static final String TAG = "LockScreenActivity";

    private TextView tvCurrentTime, tvCurrentDate;

    private TextView tVforceMode, tvRestMode, tvSwipeLock;

    private ConstraintLayout constraintRestMode, constraintNormalMode;

    private TextView tvNormalModeGameTime, tvNormalModeUseTime;

    private TextView tvRestModeResttime, tvRestModeUseTime;

    private ImageView ivCallPhone, ivMessage, ivCamera;

    private int lockMode;


    private SwipeUpFinishLayout swipeUpFinishLayout;



    public static final int TIME_CYCLE = 1000;
    private Handler handler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            tvCurrentTime.setText(TimeUtils.getTime(System.currentTimeMillis(), TimeUtils.DATE_FORMAT_HH_MM));
            tvCurrentDate.setText(TimeUtils.getDate(System.currentTimeMillis(), TimeUtils.DATE_FORMAT_MM_dd));

            handler.sendEmptyMessageDelayed(0, TIME_CYCLE);
        }
    };

    @Override

    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Window window = getWindow();
//        | WindowManager.LayoutParams.TYPE_KEYGUARD_DIALOG
        window.addFlags(WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD | WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED );

//        WindowManager windowManager = (WindowManager) getSystemService(WINDOW_SERVICE);
//
//        WindowManager.LayoutParams layoutParams = new WindowManager.LayoutParams();
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
//            layoutParams.type = WindowManager.LayoutParams.TYPE_APPLICATION_OVERLAY;
//        } else if (Build.VERSION.SDK_INT > Build.VERSION_CODES.N) {
//            layoutParams.type = WindowManager.LayoutParams.TYPE_PHONE;
//        } else {
//            layoutParams.type = WindowManager.LayoutParams.TYPE_TOAST;
//        }
//
//        layoutParams.flags = WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD | WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED | WindowManager.LayoutParams.TYPE_KEYGUARD_DIALOG | WindowManager.LayoutParams.FLAG_LAYOUT_NO_LIMITS;

        View root = LayoutInflater.from(this).inflate(R.layout.activity_lock_screen, null);


        DisplayMetrics metrics = new DisplayMetrics();


        getWindowManager().getDefaultDisplay().getRealMetrics(metrics);

//        layoutParams.width = metrics.widthPixels;


        int heightPixels = metrics.heightPixels;


//        heightPixels += getNavigationBarHeight();
//        layoutParams.height = heightPixels;


//        if (PermissionUtil.checkPermissionOpen(this, Manifest.permission.SYSTEM_ALERT_WINDOW)) {
//            windowManager.addView(root, layoutParams);
//        } else {
//
//
//        }


        ViewGroup.LayoutParams layoutParams1 = new ViewGroup.LayoutParams(metrics.widthPixels, heightPixels);

        setContentView(root, layoutParams1);

        hideStatusNavigationBar();


        findView(root);


        lockMode = LauncherManager.getInstance().getLockScreenMode();

        switch (lockMode) {
            case ColumnDef.LockScreen.MODE_NORMAL:
                setNormalModel();
                break;
            case ColumnDef.LockScreen.MODE_REST:
                setRestMode();
                break;
            case ColumnDef.LockScreen.MODE_FORCE_LOCK:
                setForceModel();
                break;
        }

        Logger.e(TAG,"onCreate"+lockMode);
    }

    @Override
    protected void onNewIntent(Intent intent) {
        if (lockMode != LauncherManager.getInstance().getLockScreenMode()) {
            lockMode = LauncherManager.getInstance().getLockScreenMode();

            switch (lockMode) {
                case ColumnDef.LockScreen.MODE_NORMAL:
                    setNormalModel();
                    break;
                case ColumnDef.LockScreen.MODE_REST:
                    setRestMode();
                    break;
                case ColumnDef.LockScreen.MODE_FORCE_LOCK:
                    setForceModel();
                    break;
            }
        }
    }

    private void findView(View root) {
        tvCurrentTime = root.findViewById(R.id.tv_current_time);
        tvCurrentDate = root.findViewById(R.id.tv_current_date);
        tVforceMode = root.findViewById(R.id.tv_force_mode);
        tvRestMode = root.findViewById(R.id.tv_rest_mode);
        constraintRestMode = root.findViewById(R.id.constraint_rest_mode);
        constraintNormalMode = root.findViewById(R.id.constraint_normal_mode);
        tvSwipeLock = root.findViewById(R.id.tv_swipe_lock);

        tvNormalModeGameTime = root.findViewById(R.id.tv_game_time);
        tvNormalModeUseTime = root.findViewById(R.id.tv_normal_mode_use_time);
        tvRestModeResttime = root.findViewById(R.id.tv_rest_time);
        tvRestModeUseTime = root.findViewById(R.id.tv_normal_use_time);
        ivMessage = root.findViewById(R.id.iv_message);
        ivCallPhone = root.findViewById(R.id.iv_call_phone);
        ivCamera = root.findViewById(R.id.iv_camera);


        swipeUpFinishLayout=new SwipeUpFinishLayout(this);

        swipeUpFinishLayout.attachActivity(this);

        ivCallPhone.setOnClickListener(this);
        ivMessage.setOnClickListener(this);
        ivCamera.setOnClickListener(this);

        tvCurrentTime.setText(TimeUtils.getTime(System.currentTimeMillis(), TimeUtils.DATE_FORMAT_HH_MM));

        tvCurrentDate.setText(TimeUtils.getDate(System.currentTimeMillis(), TimeUtils.DATE_FORMAT_MM_dd));

        handler.sendEmptyMessageDelayed(0, TIME_CYCLE);
    }


    private int getNavigationBarHeight() {
        Resources resources = getResources();
        int resourceId = resources.getIdentifier("navigation_bar_height", "dimen", "android");
        if (resourceId > 0) {
            int height = resources.getDimensionPixelSize(resourceId);
            return height;
        }
        return 0;
    }

    private void hideStatusNavigationBar() {

        int uiFlags = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION;
        getWindow().getDecorView().setSystemUiVisibility(uiFlags);

    }

    public void setRestMode() {

        tvRestMode.setVisibility(View.VISIBLE);
        tVforceMode.setVisibility(View.GONE);
        constraintRestMode.setVisibility(View.VISIBLE);

        tvSwipeLock.setVisibility(View.GONE);
        constraintNormalMode.setVisibility(View.GONE);

        tvRestModeResttime.setText(LauncherManager.getInstance().getModeDesc1());
        tvRestModeUseTime.setText(LauncherManager.getInstance().getModeDesc2());

        swipeUpFinishLayout.setEnable(false);

    }

    public void setNormalModel() {

        tvRestMode.setVisibility(View.GONE);
        tVforceMode.setVisibility(View.GONE);
        constraintRestMode.setVisibility(View.GONE);

        tvSwipeLock.setVisibility(View.VISIBLE);
        constraintNormalMode.setVisibility(View.VISIBLE);

        tvNormalModeGameTime.setText(LauncherManager.getInstance().getModeDesc1());
        tvNormalModeUseTime.setText(LauncherManager.getInstance().getModeDesc2());
        swipeUpFinishLayout.setEnable(true);
    }

    public void setForceModel() {

        tvRestMode.setVisibility(View.GONE);
        constraintRestMode.setVisibility(View.GONE);
        tvSwipeLock.setVisibility(View.GONE);
        constraintNormalMode.setVisibility(View.GONE);
        tVforceMode.setVisibility(View.VISIBLE);

        swipeUpFinishLayout.setEnable(false);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        Logger.e(TAG, "onKeyDown");
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if (LauncherManager.getInstance().isForceLock()) {
                return true;
            }
        }
        return super.onKeyDown(keyCode, event);
    }


    @Override
    public void onClick(View v) {
        Logger.e(TAG,"onClick");
        if (v==ivCallPhone){
            Intent intent=new Intent(Intent.ACTION_DIAL);
            startActivity(intent);
        }else if (v==ivCamera){
            Intent intent=new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
            startActivity(intent);
        }else if (v==ivMessage){
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.setType("vnd.android-dir/mms-sms");
            startActivity(intent);
        }
    }

    @Override
    protected void onResume() {
        ConfigManager.getInstance().setLockScreen(true);
        super.onResume();
    }

    @Override
    protected void onPause() {
        ConfigManager.getInstance().setLockScreen(false);
        super.onPause();
    }


    @Override
    protected void onDestroy() {
        handler.removeCallbacksAndMessages(null);
        super.onDestroy();
    }
}
