package com.android.launcher3.accessibility;

import android.app.Service;
import android.app.usage.UsageEvents;
import android.app.usage.UsageStats;
import android.app.usage.UsageStatsManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.IBinder;
import android.util.Log;

import com.android.launcher3.manager.ColumnDef;
import com.android.launcher3.manager.Constants;
import com.android.launcher3.manager.LauncherManager;
import com.android.launcher3.manager.ManagementInfo;
import com.android.launcher3.manager.ManagerAppProvider;
import com.android.launcher3.util.Logger;

import java.util.List;

import androidx.annotation.Nullable;

/**
 * 后台监听app的活动
 */
public class ListenerAppService extends Service {

    public static final String TAG = "ListenerAppService";

    public static final int TIME = 60 * 1000;

    private boolean active = true;

    private InnerRecevier innerReceiver;

//    private

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }


    @Override
    public void onCreate() {
        super.onCreate();


        //创建广播
        innerReceiver = new InnerRecevier();
        //动态注册广播
        IntentFilter intentFilter = new IntentFilter(Intent.ACTION_CLOSE_SYSTEM_DIALOGS);
        intentFilter.addAction(Intent.ACTION_SCREEN_OFF);
        intentFilter.addAction(Intent.ACTION_SCREEN_ON);
//启动广播
        registerReceiver(innerReceiver, intentFilter);


//        registerReceiver()

        Logger.e(TAG, "onCreate");
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {

        new Thread(new Runnable() {
            @Override
            public void run() {
                while (true) {
                    while (active) {
                        UsageStatsManager usageStatsManager = (UsageStatsManager) getSystemService(USAGE_STATS_SERVICE);

                        long currentMills = System.currentTimeMillis();

                        List<UsageStats> stats = usageStatsManager.queryUsageStats(UsageStatsManager.INTERVAL_BEST, currentMills - TIME, currentMills);

                        //取得当前在前台的app，即当前运行的app
                        if ((stats != null) && (!stats.isEmpty())) {
                            String topActivity = "";
                            int j = 0;
                            for (int i = 0; i < stats.size(); i++) {
                                if (stats.get(i).getLastTimeUsed() > stats.get(j).getLastTimeUsed()) {
                                    j = i;
                                }
                            }
                            topActivity = stats.get(j).getPackageName();

                            if ("com.android.settings".equals(topActivity)) {
                                LauncherManager.getInstance().backLauncherHome(getApplicationContext());
                                LauncherManager.getInstance().jumpSetting(getApplicationContext());
                            }


                            ManagementInfo appinfo = LauncherManager.getInstance().getManageAppInfo(topActivity);
                            if (appinfo != null) {
                                if (appinfo != null) {
                                    if (appinfo.getLock() == ColumnDef.LOCKED) {

                                        LauncherManager.getInstance().backLauncherHome(getApplicationContext(), Constants.DIALOG_TYPE_LOCKED);
                                    }
                                    if (appinfo.getLock() == ColumnDef.DEADLINE) {
                                        LauncherManager.getInstance().backLauncherHome(getApplicationContext(), Constants.DIALOG_TYPE_APP_USE_DEADLINE);

                                    }

                                    if (appinfo.getUseType() == ColumnDef.UNCLASS_TYPE) {
                                        LauncherManager.getInstance().backLauncherHome(getApplicationContext(), Constants.DIALOG_TYPE_APP_UNCLASS);
                                    }
                                }
                            }


                        }

                    }
                }
            }
        }).start();


//        usageStatsManager.
        return START_STICKY;
    }


    @Override
    public void onDestroy() {
        if (innerReceiver != null) {
            unregisterReceiver(innerReceiver);
        }
        super.onDestroy();
    }

    class InnerRecevier extends BroadcastReceiver {

        final String SYSTEM_DIALOG_REASON_KEY = "reason";

        final String SYSTEM_DIALOG_REASON_RECENT_APPS = "recentapps";

        final String SYSTEM_DIALOG_REASON_HOME_KEY = "homekey";

        @Override
        public void onReceive(Context context, Intent intent) {

            String action = intent.getAction();

            Logger.e(TAG, action);

            if (Intent.ACTION_CLOSE_SYSTEM_DIALOGS.equals(action)) {
                String reason = intent.getStringExtra(SYSTEM_DIALOG_REASON_KEY);
                if (reason != null) {
                    if (reason.equals(SYSTEM_DIALOG_REASON_HOME_KEY)) {
                        // "Home键被监听"
                        LauncherManager.getInstance().backLauncherHome(getApplicationContext());
                    }
                }
            } else if (Intent.ACTION_SCREEN_OFF.equals(action)) {
                active = false;
            } else if (Intent.ACTION_SCREEN_ON.equals(action)) {
                active = true;
            }
        }
    }
}


